#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define MAX_TERM 128
#define COEFF_LIMIT 10

void randPoly ( int f[], int n )
{
   int i;

   for (i=0; i<n; ++i)
      f[i] = rand() % (2 * COEFF_LIMIT + 1) - COEFF_LIMIT;
}

void polyPrn ( int f[], int n )
{
   int i;

   printf("(%d) * x^%d", f[n-1], n-1);
   for (i=n-2; i>=0; --i) printf("+ (%d) * x^%d", f[i], i);
   printf("\n");
}

void polyMul ( int h[], int f[], int g[], int n )
/* f and g are the input polynomials */
/* The product is to be stored in the polynomial h */
/* n is the number of terms (not the degree) in each of the input polynomials */
{
   int m, i;
   int f1[MAX_TERM], f0[MAX_TERM];     /* Local copies of two halves of f */
   int g1[MAX_TERM], g0[MAX_TERM];     /* Local copies of two halves of g */
   int f0g0[MAX_TERM], f1g1[MAX_TERM]; /* Local storage for f1g0 and f1g1 */
   int f1f0[MAX_TERM], g1g0[MAX_TERM]; /* Local storage for f1+f0 and g1+g0 */
   int f1f0g1g0[MAX_TERM];             /* Local storage for (f1+f0)(g1+g0) */

   /* Basis case: Product of constant polynomials */
   if (n == 1) { h[0] = f[0] * g[0]; return; }

   /* Pad with zero to make the number of terms even */
   if (n % 2 == 1) { f[n] = g[n] = 0; ++n; }

   /* m is half of the number of terms of the input polynomials */
   m = n / 2;

   /* Make local copies of the half polynomials */
   for (i=0; i<m; ++i) {
      f1[i] = f[m+i]; g1[i] = g[m+i];
      f0[i] = f[i]; g0[i] = g[i];
   }

   /* First recursive call: lo by lo */
   polyMul(f0g0, f0, g0, m);

   /* Second recursive call: hi by hi */
   polyMul(f1g1, f1, g1, m);

   /* Compute f1+f0 and g1+g0 */
   for (i=0; i<m; ++i) {
      f1f0[i] = f1[i] + f0[i];
      g1g0[i] = g1[i] + g0[i];
   }

   /* Third recursive call: hi by lo */
   polyMul(f1f0g1g0, f1f0, g1g0, m);

   /* Initialize h to zero */
   for (i=0; i<=4*m-2; ++i) h[i] = 0;

   /* Add/subtract the products of half polynomials at appropriate places */
   for (i=0; i<=2*m-2; ++i) {
      h[i] += f0g0[i];
      h[m+i] += f1f0g1g0[i] - f0g0[i] - f1g1[i];
      h[2*m+i] += f1g1[i];
   }
}

int main ()
{
   int f[MAX_TERM], g[MAX_TERM], h[MAX_TERM], d;

   srand((unsigned int)time(NULL));
   printf("Enter degree (<= %d): ", (MAX_TERM / 2) - 1); scanf("%d", &d);
   randPoly(f,d+1); printf("f  = "); polyPrn(f,d+1);
   randPoly(g,d+1); printf("g  = "); polyPrn(g,d+1);
   polyMul(h,f,g,d+1); printf("fg = "); polyPrn(h,2*d+1);
   printf("fg - f*g\n");

   exit(0);
}
